/*
* balancer.js - Script for reading personal info, calculating statistics and saving change log 
*/

var debug = false; // debug mode

var waitTimeout = 60000; // timeout in millisec
var sleepPeriod = 200; // Sleep period

// FileSystemObject constants
var FOR_READING = 1,
    FOR_WRITING = 2,
    FOR_APPENDING = 8;
var READYSTATE_UNINITIALIZED = 0,
    READYSTATE_LOADING = 1,
    READYSTATE_LOADED = 2,
    READYSTATE_INTERACTIVE = 3,
    READYSTATE_COMPLETE = 4;


var log = "";
printMes("==========================================================", true);
printMes("", true);
printMes(WScript.ScriptName + " - start");
printMes("", true);

// Init Objects

// WScript Object Properties and Methods
// http://msdn.microsoft.com/en-us/library/2795740w%28v=vs.85%29.aspx
var oShell = new ActiveXObject("WScript.Shell");
// http://www.java2s.com/Tutorial/JavaScript/0600__MS-JScript/0220__FileSystemObject.htm
var oFSO = new ActiveXObject("Scripting.FileSystemObject");

var scriptName = WScript.ScriptName.replace(".js", "");

var accounts = parseCommandLine();

var plugins = findPlugins();

// http://msdn.microsoft.com/en-us/library/aa752084%28v=vs.85%29.aspx
var oIE = new ActiveXObject("InternetExplorer.Application");
oIE.Navigate("about:blank");
oIE.Visible = debug;

// running plugins
var rezults = new Array();
var accPos = -1;
for (accPos = 0; accPos < accounts.length; accPos++) {
    if (plugins[accounts[accPos][0]] != null) {
        // rezults[] =  |  |  |  | . . | .  
        var str = "rezults.push("
            + accounts[accPos][0] + "('" + accounts[accPos][2] + "', '"
            + accounts[accPos][3] + "')); " + plugins[accounts[accPos][0]][1];
        printMes("", true);
        try {
            printMes(accounts[accPos][0] + " " + accounts[accPos][1] + " - start");
            eval(str);
            rezults[accPos][0] = accounts[accPos][0] + " " + accounts[accPos][1];
        } catch (e) {
            printMes("ERROR in 'running plugins' (" +
                accounts[accPos][0] + " " + accounts[accPos][1] +
                "): (" + e.number + ") " + e.name + " - " + e.message + " (" + e.description + ")");
        }
    } else {
        printMes("", true);
        printMes("WARNING: Plugin '" + accounts[accPos][0] + "' is not find");
    }
}

if (!debug) {
    oIE.Stop();
    oIE.Quit();
    oIE = null;
}

if (rezults.length > 0) {
    checkBalChanges();
    generateReport();
} else {
    printMes("WARNING: Empty rezult...");
}
printMes("", true);
printMes(WScript.ScriptName + " - stop");
printMes("", true);
printMes("==========================================================", true);
scriptQuit();
//writeFile(scriptName + ".log", log);

function waitRefresh() {
    try {
        WScript.Sleep(sleepPeriod);
        var startTime = new Date().getTime();
        while ((oIE.Busy || oIE.ReadyState != READYSTATE_COMPLETE)
                && (new Date().getTime() < (startTime + waitTimeout))) {
            WScript.Sleep(sleepPeriod);
        }
        if (oIE.Busy || oIE.ReadyState != READYSTATE_COMPLETE) {
            oIE.Stop();
            if (new Date().getTime() >= (startTime + waitTimeout)) {
                printMes("WARNING in 'waitRefresh': request timeout");
            } else {
                printMes("WARNING in 'waitRefresh': IE is busy again");
            }
            return false;
        } else {
            return true;
        }
    } catch (e) {
        printMes("ERROR in 'waitRefresh': " + e.name + " - " + e.message);
    }
    return false;
}

function waitPageContent(regStr, fildNum) {
    try {
        if (waitRefresh()) {
            var startTime = new Date().getTime();
            var oRE = new RegExp(regStr, "gim");
            while (new Date().getTime() < (startTime + waitTimeout)) {
                //        WScript.Echo("waitPageContent - Start: " + (startTime + waitTimeout) + "; Current: " + new Date().getTime());
                if (oIE.document && oIE.document.documentElement) {
                    var html = oIE.document.documentElement.outerHTML;
                    var aRes = oRE.exec(html);
                    if (aRes && aRes.length > fildNum - 1) {
                        return true;
                    }
                }
                WScript.Sleep(sleepPeriod);
            }
            oIE.Stop();
            printMes("WARNING in 'waitPageContent': request timeout");
//            saveHtml();
        }
    } catch (e) {
        printMes("ERROR in 'waitPageContent': " + e.name + " - " + e.message);
//        saveHtml();
    }
    return false;
}

function waitTitle(title) {
    try {
        var startTime = new Date().getTime();
        while (!oIE.document.title.toString().match(".*" + title + ".*")
                && (new Date().getTime() < (startTime + waitTimeout))) {
            WScript.Sleep(sleepPeriod);
        }
        if (oIE.document.title.toString().match(".*" + title + ".*")) {
            return true;
        } else {
            printMes("WARNING in 'waitTitle': request timeout");
        }
    } catch (e) {
        printMes("ERROR in 'waitTitle': " + e.name + " - " + e.message);
    }
    return false;
}

function waitIframeContent(framePos, regStr) {
    try {
        if (waitRefresh()) {
            var html = "";
            var startTime = new Date().getTime();
            var oRE = new RegExp(regStr, "gim");
            while (new Date().getTime() < (startTime + waitTimeout)) {
                //        WScript.Echo("waitPageContent - Start: " + (startTime + waitTimeout) + "; Current: " + new Date().getTime());
                var oIframe = oIE.document.getElementsByTagName("iframe")[framePos];
                if (oIframe) {
                    var oDoc = oIframe.contentWindow || oIframe.contentDocument;
                    if (oDoc.document) {
                        html = oDoc.document.documentElement.outerHTML;
                        return oRE.exec(html);
                    }
                }
                WScript.Sleep(sleepPeriod);
            }
            oIE.Stop();
            printMes("WARNING in 'waitFrameContent': request timeout");
            var logName = getDataFileName(accPos);
            printMes("Save iframe content to '" + logName + "_frame" + framePos + ".htm'");
            writeFile(logName + "_frame" + framePos + ".htm", html);
        }
    } catch (e) {
        printMes("ERROR in 'waitFrameContent': " + e.name + " - " + e.message);
        writeFile(logName + "_frame" + framePos + ".htm", oIE.document.documentElement.outerHTML);
    }
    return false;
}

function waitIframe(framePos) {
    try {
        if (waitRefresh()) {
            var html = "";
            var startTime = new Date().getTime();
            while (new Date().getTime() < (startTime + waitTimeout)) {
                var oIframe = oIE.document.getElementsByTagName("iframe")[framePos];
                if (oIframe) {
                    var oDoc = oIframe.contentWindow || oIframe.contentDocument;
                    if (oDoc.document) {
                        return oDoc;
                    }
                }
                WScript.Sleep(sleepPeriod);
            }
            oIE.Stop();
            printMes("WARNING in 'waitIframe': request timeout");
            var logName = getDataFileName(accPos);
            printMes("Save iframe content to '" + logName + "_frame" + framePos + ".htm'");
            writeFile(logName + "_frame" + framePos + ".htm", html);
        }
    } catch (e) {
        printMes("ERROR in 'waitFrameContent': " + e.name + " - " + e.message);
        writeFile(logName + "_frame" + framePos + ".htm", oIE.document.documentElement.outerHTML);
    }
    return null;
}

function navigateAndWait(URI) {
    if (URI.length > 1 && oIE) {
        oIE.Navigate(URI);
        return waitRefresh();
    }
    return false;
}

function getElementsByAttrVal(elemname, attrname, attrval, root) {
    if (root == null) {
        root = oIE.document.getElementsByTagName("body")[0];
    }
    var rez = [];
    var nodes = root.getElementsByTagName(elemname);
    for (var i = 0, j = nodes.length; i < j; i++) {
        //        printMes("node[" + i + "]: " + nodes[i].outerHTML, true);
        if (attrname == "class") {
            if (nodes[i].className == attrval) {
                rez.push(nodes[i]);
                //                printMes("Find!", true);
            }
        } else {
            var attr = nodes[i].getAttribute(attrname);
            if (attr != null && attr == attrval) {
                rez.push(nodes[i]);
                //                printMes("Find!", true);
            }
        }
    }
    return rez;
}

function getFirstElemByAttrVal(elemname, attrname, attrval, root) {
    if (root == null) {
        root = oIE.document.getElementsByTagName("body")[0];
    }
    var nodes = root.getElementsByTagName(elemname);
    for (var i = 0, j = nodes.length; i < j; i++) {
        if (attrname == "class") {
            if (nodes[i].className == attrval) {
                return nodes[i];
            }
        } else {
            var attr = nodes[i].getAttribute(attrname);
            if (attr != null && attr == attrval) {
                return nodes[i];
            }
        }
    }
    return null;
}

/* 
* Function for reading plugins
* output: plugins[hashMap] = Plugin name | Plugin content
*/
//function readPlugins() {
//    var plugins = new Array();
//    if (!oFSO.FolderExists("plugins")) {
//        printMes("WARNING: The folder 'plugins' is not found!", false);
//        scriptQuit();
//    }
//    var oDir = oFSO.GetFolder("plugins");
//    if (oDir) {
//        var oEnum = new Enumerator(oDir.Files);
//        while (!oEnum.atEnd()) {
//            var oFile = oEnum.item();
//            if (oFSO.GetExtensionName(oFile.Name).toLowerCase() == "js") {
//                var plug_name = oFile.Name.replace(".js", "").toUpperCase();
//                if (findAccountByName(plug_name) != null) {
//                    printMes("Load plugin - '" + plug_name + "'");
//                    var plugin_txt = readFile(oFile.ShortPath);
//                    if (plugin_txt.length > 0) {
//                        plugins[plug_name] = plugin_txt;
//                    }
//                }
//            }
//            oEnum.moveNext();
//        }
//    }
//    if (!plugins) {
//        printMes("WARNING: Operator plugins list is empty!", false);
//        scriptQuit();
//    }
//    return plugins;
//}

/*
* Function for reading plugins
* output: plugins[hashMap] = Plugin name | Plugin content
*/
function findPlugins() {
    var plugins = new Array();
    var oDir = oFSO.GetFolder(".");
    if (oDir) {
        findPluginsInFolder(oDir, plugins);
    }
    if (!plugins) {
        printMes("WARNING: Operator plugins list is empty!", false);
        scriptQuit();
    }
    return plugins;
}

function findPluginsInFolder(rootFolder, plugins) {
    var oEnum = new Enumerator(rootFolder.Files);
    while (!oEnum.atEnd()) {
        var oFile = oEnum.item();
        if (oFSO.GetExtensionName(oFile.Name).toLowerCase() == "js") {
            var plug_name = oFile.Name.replace(".js", "").toUpperCase();
            if (plugins[plug_name] == null
                    && findAccountByName(plug_name) != null) {
                // WScript.Echo("Load plugin - '" + plug_name + "'");
                var plugin_txt = readFile(oFile.ShortPath);
                if (plugin_txt.length > 0) {
                    plugins[plug_name] = new Array(2);
                    plugins[plug_name][0] = oFile.ParentFolder;
                    plugins[plug_name][1] = plugin_txt;
                    if (debug) {
                        printMes("Load plugin - " + oFile.ParentFolder, false);
                    } else {
                        printMes("Load plugin - '" + plug_name + "'", false);
                    }
                }
            }
        }
        oEnum.moveNext();
    }
    oEnum = new Enumerator(rootFolder.SubFolders);
    while (!oEnum.atEnd()) {
        var oDir = oEnum.item();
        findPluginsInFolder(oDir, plugins);
        oEnum.moveNext();
    }
}

function findAccountByName(name) {
    for (var i = 0; i < accounts.length; i++) {
        if (accounts[i][0] == name) {
            return accounts[i];
        }
    }
    return null;
}

function writeFile(sFile, sCont) {
    if (!sCont) sCont = "";
    var oTF = oFSO.CreateTextFile(sFile, true);
    oTF.Write(sCont);
    oTF.Close();
    return oFSO.GetAbsolutePathName(sFile);
}

function readFile(sFile) {
    if (oFSO.FileExists(sFile)) {
        var oTF = oFSO.OpenTextFile(sFile, FOR_READING);
        var sCont = oTF.ReadAll();
        oTF.Close();
        return sCont;
    } else {
        printMes("WARNING in 'readFile': file is not found - '" + sFile + "'");
    }
    return "";
}

function readLastDataFromFile(sFile) {
    if (oFSO.FileExists(sFile)) {
        var oTF = oFSO.OpenTextFile(sFile, FOR_READING);
        var sCont = "";
        if (!oTF.AtEndOfStream) {
            sCont = oTF.ReadLine();
        }
        oTF.Close();
        return sCont;
    } else {
        printMes("WARNING in 'readLastDataFromFile': file is not found - '" + sFile + "'");
    }
    return "";
}

function printMes(mes, noCT) {
    if (!noCT) {
        mes = getCurTime() + " " + mes;
    }
    WScript.Echo(mes);
    log += mes + "\r\n";
}

function errMes(err, mes) {
    printMes("Error: " + mes);
    var rez = err;
    if (rez != null && rez.length > 0) {
        rez += "<br>";
    } else {
        rez = "ERROR: ";
    }
    return (rez + mes);
}

function findErrMesInHTML(errMes) {
    if (oIE.document && oIE.document.documentElement
            && oIE.document.documentElement.outerHTML.length > 10) {
        if (oIE.document.title != null && oIE.document.title.length > 0) {
            return oIE.document.title.substring(0, 50);
        }
        if (oIE.document.body != null && oIE.document.body.innerText.length > 0) {
            return oIE.document.body.innerText.substring(0, 50);
        }
        return oIE.document.documentElement.innerText.substring(0, 50);
    }
    return (errMes != null && errMes.length > 0) ? errMes : "The personal page is not load or markup is changed";
}

function insertLinkToLastLoadHTML(errMes) {
    if (errMes.length > 0) {
        var htmlFile = saveHtml();
        if (htmlFile) {
            return "<a href='" + htmlFile + "' target='_blank'>" + errMes + "</a>";
        }
    }
    return errMes;
}

function trim(string) {
    return string.replace(/(^\s+)|(\s+$)/g, "");
}

function getCurTime() {
    var now = new Date();
    var hours = now.getHours();
    var minutes = now.getMinutes();
    var seconds = now.getSeconds();
    var millisec = now.getMilliseconds();
    var sTime = new String();
    sTime = ((hours < 10) ? "0" : "") + hours;
    sTime += ((minutes < 10) ? ":0" : ":") + minutes;
    sTime += ((seconds < 10) ? ":0" : ":") + seconds;
    sTime += ".";
    if (millisec < 10) {
        sTime += "00";
    } else if (millisec < 100) {
        sTime += "0";
    }
    sTime += millisec;
    return sTime;
}

function getCurDate() {
    var now = new Date();
    var year = now.getYear();
    var month = now.getMonth() + 1;
    var day = now.getDate();
    var date = year;
    date += ((month < 10) ? "/0" : "/") + month;
    date += ((day < 10) ? "/0" : "/") + day;
    return date;
}

// calculate days to date
// input param 'destdate' must be in format - 'YYYY/MM/DD'
function calcDaysToDate(destdate) {
    var adest = destdate.split("/");
    if (adest != null && adest.length == 3) {
        var ddest = new Date(adest[0], adest[1] - 1, adest[2]);
        return Math.ceil((ddest - new Date()) / (1000 * 60 * 60 * 24));
    }
    return null;
}

function generateReport() {
    var sHTML = new String("");
    var sHTML1 = new String("");
    var sHTMLTitle = new String("");
    sHTML = "<!DOCTYPE html PUBLIC \'-//W3C//DTD XHTML 1.0 Transitional//EN\'"
        + " \'http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\'>";
    sHTML += "\r\n<html xmlns=\'http://www.w3.org/1999/xhtml\'>";
    sHTML += "\r\n<head><meta http-equiv=\'content-type\' content=\'text/html; charset=windows-1251\' />";
    sHTML += "\r\n<style type='text/css'>table{border-bottom:2px solid aqua;border-right:2px solid aqua;}" +
            "td{border-top:2px solid aqua;border-left:2px solid aqua;vertical-align:top;white-space:nowrap;}" +
            "a{text-decoration:none;}</style>";

    sHTML += "\r\n<title>JSBalancer (" + getCurDate() + " " + getCurTime()
        + ")</title></head>\r\n<body>\r\n<table>";
    sHTML += "\r\n<tr style='color: blue;'><td><b></b></td>"
        + "<td><b></b></td><td><b>. .</b></td><td><b></b></td>"
        + "<td> </td><td><b title='   '>.</b></td><td><b title='  '>/</b></td><td><b title='  '>/</b></td></tr>";
    for (var i = 0; i < rezults.length; i++) {
        var filds = rezults[i].length;
        for (var j = 0; j < rezults[i].length; j++) {
            if (!rezults[i][j]) rezults[i][j] = "";
        }
        sHTML += "\r\n<tr><td>";    // 
        if (rezults[i][5].length > 0)
            sHTML += "<a href='" + rezults[i][5] + "' target='_blank'>";  //    
        sHTML += rezults[i][0];    // 
        if (filds > 10 && rezults[i][5].length > 0) sHTML += "</a>";
        sHTML += "</td>"   // 
                + "<td>" + rezults[i][3] + "</td>";           // 
        //                + "<td";           // . .
        sHTML += "<td>" + rezults[i][4] + "</td>"          // . .
                + "<td style='text-align:right;'>" + rezults[i][1] + "</td>"    // 
                + "<td style='text-align:center;'>" + rezults[i][2] + "</td>";    // 
        if (filds > 10) {
            sHTML += "<td style='color:" + getChangeTextColor(rezults[i][filds - 5]) + ";text-align:right;'>" + rezults[i][filds - 5] + "</td>"    // 
                + "<td style='color:" + getChangeTextColor(rezults[i][filds - 4]) + ";text-align:right;'>" + rezults[i][filds - 4] + "</td>"    //   
                + "<td style='color:" + getChangeTextColor(rezults[i][filds - 3]) + ";text-align:right;' title='+" + rezults[i][filds - 1]
                + "/" + rezults[i][filds - 2] + "'>" + rezults[i][filds - 3] + "</td>";     //   .
        } else {
            sHTML += "<td></td>"    // 
                + "<td></td>"    //   
                + "<td></td>"    //   .
        }
        sHTML += "</tr>";
    }
    sHTML += "\r\n</table>\r\n</body></html>";
    oShell.Run("iexplore.exe " + writeFile(scriptName + ".htm", sHTML));
}

function getChangeTextColor(ival) {
    var color = "blue";
    if (ival > 0) {
        color = "green";
    } else if (ival < 0) {
        color = "red";
    }
    return color;
}

function checkBalChanges() {
    for (var i = 0; i < rezults.length; i++) {

        var dataFile = getDataFileName(i) + ".dat";
        var curBal = rezults[i][1];
        if (isNumber(curBal)) {
            var lastDat = getDataFromFile(dataFile);
            var save = true;
            var val = new Array(0, 0, 0, 0, 0);
            if (lastDat && lastDat.length > 2) {
                val[0] = curBal - lastDat[0];
                if (val[0] == 0) save = false;
                if (isNumber(lastDat[1])) {
                    val[1] = curBal - lastDat[1];
                }
                if (isNumber(lastDat[2])) {
                    val[2] = curBal - lastDat[2];
                }
                if (isNumber(lastDat[3]) && isNumber(lastDat[4])) {
                    val[3] = lastDat[3];
                    val[4] = lastDat[4];
                    var rash = curBal - lastDat[0];
                    if (rash < 0) {
                        val[3] += rash;
                    } else if (rash > 0) {
                        val[4] += rash;
                    }
                }
            }
            for (var j = 0; j < val.length; j++) {
                rezults[i].push(val[j].toFixed(2));
            }
            //            for (var j = 0; j < rezults[i].length; j++) {
            //                WScript.Echo("rezults[" + i + "][" + j + "] = " + rezults[i][j]);
            //            }

            if (save) {
                lastDat = new Array(2);
                lastDat[0] = getCurDate();
                lastDat[1] = curBal;
                saveCurrentData(dataFile, lastDat);
            }
        }
    }
}

function getDataFromFile(sFile) {
    if (oFSO.FileExists(sFile)) {
        var rez = new Array();
        var line;
        var oTF = oFSO.OpenTextFile(sFile, FOR_READING);
        var arr = new Array();
        while (!oTF.AtEndOfStream) {
            var sCont = oTF.ReadLine();
            var line = sCont.split("\t");
            arr.push(line[1]);
            if (!rez[0]) {
                rez[0] = line[1];
                //                WScript.Echo("fill rez[0] = " + line[1]);
            }
            var ldate = line[0].split("/");
            if (!rez[1] && ldate[2] != new Date().getDate()) {
                rez[1] = line[1];
                //                WScript.Echo("fill rez[1] = " + line[1]);
            }
            if (!rez[2] && ldate[1] != new Date().getMonth() + 1) {
                rez[2] = line[1];
                //                WScript.Echo("fill rez[2] = " + line[1]);
                break;
            }
        }
        oTF.Close();
        if (line && line.length > 1) {
            //            WScript.Echo("");
            for (var i = 0; i < 3; i++) {
                //                WScript.Echo("rez[" + i + "] = " + rez[i]);
                if (!rez[i]) rez[i] = line[1];
            }
        }
        var arr1 = new Array(0, 0);
        if (arr.length > 0) {
            for (var i = arr.length - 2; i > -1; i--) {
                var val = arr[i] - arr[i + 1];
                if (val < 0) {
                    arr1[0] += val; // rashod
                } else if (val > 0) {
                    arr1[1] += val;  // prihod
                }
            }
        }
        //        WScript.Echo("rashod: " + arr1[0]);
        //        WScript.Echo("prihod: " + arr1[1]);
        rez.push(arr1[0]); // rashod
        rez.push(arr1[1]); // prihod
        //        for (var i = 0; i < rez.length; i++) {
        //            WScript.Echo("rez[" + i + "] = " + rez[i]);
        //        }
        return rez;
    } else {
        printMes("WARNING in 'getDataFromFile': file is not found - '" + sFile + "'");
    }
    return null;
}

function isNumber(o) {
    if (o == null || o.length == 0) return false;
    return !isNaN(o - 0);
}

function normalizeNumVal(val) {
    val = val.replace(" ", ""); //  
    val = val.replace("&nbsp;", ""); //  
    val = val.replace("\xA0", ""); //  
    val = val.replace(",", "."); //    
    return (isNumber(val) ? val : null);
}

function getLastData(dataFile) {
    if (oFSO.FileExists(dataFile)) {
        var str = readLastDataFromFile(dataFile);
        var arr = str.split("\t");
        return arr;
    }
    return null;
}

function saveCurrentData(dataFile, lastDat) {
    if (lastDat.length > 1) {
        var str = readFile(dataFile);
        if (str.length > 0) str = "\r\n" + str;
        str = lastDat[0] + "\t" + lastDat[1] + str;
        return writeFile(dataFile, str);
    }
    return null;
}

function getHtml() {
    var html = null;
    if (oIE.document && oIE.document.documentElement) {
        html = oIE.document.documentElement.outerHTML;
        if (!html || html.length == 0) {
            printMes("WARNING: HTML is empty");
        }
        if (debug) {
            writeFile(getDataFileName(accPos) + ".htm", html);
        }
    }
    return html;
}

function saveHtml() {
    //    printMes("saveHtml: oIE.document = " + (oIE.document != null));
    //    printMes("saveHtml: oIE.document.documentElement = " + (oIE.document.documentElement != null));
    //    printMes("saveHtml: fileName = " + (fileName != null));
    //    printMes("saveHtml: fileName.length = " + (fileName.length));
    var fileName = getDataFileName(accPos) + ".htm";
    if (oIE.document && oIE.document.documentElement) {
        printMes("Save page HTML to '" + fileName);
        return writeFile(fileName, oIE.document.documentElement.outerHTML);
    }
    return null;
}

function getDataFileName(pos) {
    var fileName = "test";
    if (pos != null && pos > -1) {
        fileName = plugins[accounts[pos][0]][0] + "/" + accounts[pos][0] + "_" + accounts[pos][1];
    }
    return fileName;
}

function parseCommandLine() {
    var objArgs = WScript.Arguments;
    var arr = null;
    if (objArgs != null) {
        var argsLenth = objArgs.length;
        if (argsLenth == 2 || argsLenth == 5) {
            var lastVal = objArgs(argsLenth - 1);
            lastVal = lastVal.toString().toLowerCase();
            if (lastVal == "true" || lastVal == "false") {
                argsLenth--;
                //                debug = objArgs(argsLenth - 1);
                debug = (lastVal == "true") ? true : false;
            }
        }
        if (argsLenth == 1) {
            var fileName = objArgs(0);
            arr = getAccountsFromCriptFile(fileName);
            if (!arr || arr.length < 1)
                arr = getAccounts(fileName);
        } else if (argsLenth == 4) {
            arr = new Array(1);
            var arr1 = new Array(3);
            for (var i = 0; i <= 3; i++) {
                arr1[i] = objArgs(i);
            }
            arr[0] = arr1;
            scriptName = objArgs(0).toString().toLowerCase();
            //        printMes("test> " + arr[0][0] + " " + arr[0][1] + " " + arr[0][2] + " " + arr[0][3]);
        }
    }
    if (!arr || arr.length < 1) {
        printMes("WARNING: Empty account list!", false);
        scriptQuit();
    }
    return arr;
}

/*
* Function for reading accounts info from file
* output: arr[] = {Plugin name, User name, Login, Password}
*/
function getAccounts(fileName) {
    var arr;
    var cont = readFile(fileName);
    if (cont.length > 0) {
        var arr1 = cont.split("\r\n");
        arr = new Array(arr1.length);
        for (var i = 0; i < arr1.length; i++) {
            var arr2 = arr1[i].split("\t");
            if (arr2.length != 4) {
                arr = null;
                break;
            } else {
                arr[i] = arr2;
            }
        }
    }
    if (arr && arr.length > 0) {
        return arr;
    }
    return arr;
}

function getAccountsFromCriptFile(fileName) {
    // WARNING!!! acnts.bal file must be only ASCII file
    var cont = readFile(fileName);
    if (cont.length > 0 && Math.round(cont.length / 64) == cont.length / 64) {
        var val = cont.length / 64 / 4;
        var arr = new Array(val);
        var pos = 0;
        var x = 0, y = 0;
        while (pos + 64 <= cont.length) {
            if (x == 0) arr[y] = new Array(4);
            var val = getPasFromStr(cont.substr(pos, 64));
            arr[y][x] = val;
            pos += 64; x++;
            if (x > 3) {
                y++; x = 0;
            }
        }
        if (arr && arr.length > 0) {
            return arr;
        }
    }
    return null;
}

function getPasFromStr(inStr) {
    var len_pas = inStr.charCodeAt(inStr.length - 1);
    var shift2 = inStr.charCodeAt(inStr.length - 2);
    var shift1 = inStr.charCodeAt(inStr.length - 3);
    var val = Math.floor((inStr.length - len_pas) / 2);
    var pas_str = inStr.substr(val, len_pas);
    var outStr = "";
    for (var i = 0; i < pas_str.length; i++) {
        var code = pas_str.charCodeAt(i) + shift2;
        outStr += String.fromCharCode(code);
    }
    pas_str = outStr;
    outStr = pas_str.substr(shift1, pas_str.length)
            + pas_str.substr(0, shift1);
    pas_str = outStr;
    return pas_str;
}

function encode64(input) {
    var keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
    var StringMaker = function() {
        this.parts = [];
        this.length = 0;
        this.append = function(s) {
            this.parts.push(s);
            this.length += s.length;
        }
        this.prepend = function(s) {
            this.parts.unshift(s);
            this.length += s.length;
        }
        this.toString = function() {
            return this.parts.join('');
        }
    }
    var output = new StringMaker();
    var chr1, chr2, chr3;
    var enc1, enc2, enc3, enc4;
    var i = 0;
    while (i < input.length) {
        chr1 = input.charCodeAt(i++);
        chr2 = input.charCodeAt(i++);
        chr3 = input.charCodeAt(i++);
        enc1 = chr1 >> 2;
        enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
        enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
        enc4 = chr3 & 63;
        if (isNaN(chr2)) {
            enc3 = enc4 = 64;
        } else if (isNaN(chr3)) {
            enc4 = 64;
        }
        output.append(keyStr.charAt(enc1) + keyStr.charAt(enc2) + keyStr.charAt(enc3) + keyStr.charAt(enc4));
    }
    return output.toString();
}

function scriptQuit() {
    //    writeFile(WScript.ScriptName.replace(".js", "") + ".log", log);
    writeFile(scriptName + ".log", log);
    WScript.Quit(1);
}

// jsbalancer.js vers: 1.0.8 (15:59 13.11.2011)	 Voronkov Nikolay	jsbalancer@yandex.ru    http://jsbalancer.narod.ru
